'''
Created on Apr 22, 2013

@author: Christian M Brodbeck
'''
import numpy as np
from scipy.io import savemat
import wx

from .._wxgui import get_app


def draw_text(text, face='Scheherazade', size=42, spo2=False, w=None, h=None,
              fg=None, bg=None):
    """
    Use wxPython's text rendering engine to convert unicode text to an image
    in the form of a numpy array.

    Parameters
    ----------
    text : unicode
        The text.
    face : str
        The font name.
    size : int
        Font size.
    spo2 : bool
        If the shape is determined automatically (with shape=None), use a
        Square with a side length that its a Power Of 2.
    w, h : None | int
        Specify the desired width and/or height for the output array.
        If None, the shape is determined based on the text size.
    fg : 0-255 RGB tuple
        Text foreground color.
    bg : 0-255 RGB tuple
        Text background color.

    Returns
    -------
    array : (h, w, 3) | (h, w)
        If fg and/or bg are specified an RGB array, otherwise a monochromatic
        array.
    """
    if (w is not None or h is not None) and spo2:
        err = ("spo2 can not be true when providing custom width and/ir "
               "height values.")
        raise TypeError(err)
    mono = not (fg or bg)

    get_app()
    font = wx.Font(size, family=wx.FONTFAMILY_UNKNOWN, style=wx.FONTSTYLE_NORMAL,
                   weight=wx.FONTWEIGHT_NORMAL, face=face)

    # device context for drawing
    dc = wx.MemoryDC()
    dc.SetFont(font)

    # determine scales using temporary bitmap
    res = 512
    dc.SelectObject(wx.EmptyBitmap(res, res))
    tw, th = dc.GetTextExtent(text)
    if w is None:
        w = tw
    if h is None:
        h = th

    if spo2:
        x = 2
        while (x < w) or (x < h):
            x *= 2
        w, h = x, x
    elif tw > w or th > h:
        err = ("Text dimensions (w=%i, h=%i) exceed image resolution "
               "(w=%i, h=%i)." % (tw, th, w, h))
        raise ValueError(err)

    # draw in nea bitmap
    bmp = wx.EmptyBitmap(w, h)
    dc.SelectObject(bmp)
    if fg is not None:
        dc.SetTextForeground(fg)
    if bg is not None:
        dc.SetTextBackground(bg)
        brush = wx.Brush(bg, wx.SOLID)
        dc.SetBackground(brush)
    dc.Clear()

    dc.DrawText(text, (w - tw) / 2, (h - th) / 2)

    im = bmp.ConvertToImage()
    a = np.fromstring(im.GetData(), dtype='uint8').reshape((h, w, 3))
    if mono:
        return 255 - a[:, :, 0]
    else:
        return a


def draw_paragraph(lines, face='Scheherazade', size=42, align='center'):
    """
    Parameters
    ----------
    lines : iterator
        Iterator over unicode strings, each of which will be added as a
        separate line.
    face : str
        The font name.
    size : int
        Font size.
    align : 'left' | 'center' | 'right'
        How to align lines
    """
    ims = [draw_text(line, face=face, size=size, spo2=False) for line in lines]
    shape = np.array([im.shape for im in ims])
    h = np.sum(shape[:, 0])
    w = np.max(shape[:, 1])
    out = np.zeros((h, w), dtype='uint8')
    y = 0
    for im in ims:
        im_h, im_w = im.shape
        y0 = y
        y += im_h
        if align == 'left':
            x = 0
            x1 = im_w
        elif align == 'center':
            x = (w - im_w) // 2
            x1 = x + im_w
        elif align == 'right':
            x = w - im_w
            x1 = w
        else:
            raise ValueError("align=%r" % align)

        out[y0:y, x:x1] = im

    return out


class UnicodeImages(object):
    """
    Build a dictionary with image arrays depicting unicode strings, ensuring
    the same string is not stored twice.
    """
    def __init__(self, face='Scheherazade', size=42, spo2=False):
        """
        Parameters
        ----------
        draw_text parameters
        """
        self.arrays = {}
        self._codes = {}
        self._face = face
        self._size = size
        self._spo2 = spo2

    def code(self, text):
        """
        Retrieve the code for the text.

        Parameters
        ----------
        text : unicode
            The unicode string to be converted to an image.

        Returns
        -------
        code : int
            The code assigned to the unicode object.
        """
        if text in self._codes:
            code = self._codes[text]
        else:
            im = draw_text(text, face=self._face, size=self._size,
                           spo2=self._spo2)
            code = len(self._codes)
            str_code = 'i%i' % code
            self.arrays[str_code] = im
            self._codes[text] = code

        return code

    def save(self, fname):
        "Save as matlab *.m file."
        savemat(fname, self.arrays, do_compression=True)



'''
Created on Jun 17, 2012


http://tkinter.unpythonic.net/wiki/tkFileDialog


@author: Christian M Brodbeck
'''
import logging
import tkFileDialog
from tkFileDialog import Open, SaveAs
import tkMessageBox
from Tkinter import Tk



def ask_saveas(title, message, filetypes, defaultDir=None, defaultFile=None):
    dlg = SaveAs(title=title, message=message, filetypes=filetypes)
    filename = dlg.show()
    return filename


def ask_dir(title="Select Folder",
            message="Please Pick a Folder",
            must_exist=True):
    return tkFileDialog.askdirectory(title=title, mustexist=must_exist)


def ask_file(title, message, filetypes, directory, mult):
    dlg = Open(title=title, filetypes=filetypes, initialdir=directory,
               multiple=mult)
    out = dlg.show()
    return out


def ask(title="Overwrite File?",
        message="Duplicate filename. Do you want to overwrite?",
        cancel=False,
        default=True,  # True=YES, False=NO, None=Nothing
        ):
    return tkMessageBox.askyesno(title, message)


def ask_str(message, title, default=''):
    return Tk.tkSimpleDialog.askstring(title, message, initialvalue=default)


def copy_file(path):
    raise NotImplementedError


def copy_text(text):
    # http://stackoverflow.com/a/4203897/166700
    r = Tk()
    r.withdraw()
    r.clipboard_clear()
    r.clipboard_append(text)
    r.destroy()


def message(title, message="", icon='i'):
    if icon in 'i?':
        tkMessageBox.showinfo(title, message)
    elif icon == '!':
        tkMessageBox.showwarning(title, message)
    elif icon == 'error':
        tkMessageBox.showerror(title, message)
    else:
        raise ValueError("Invalid icon argument: %r" % icon)


class progress_monitor:
    def __init__(self, i_max=None,
                 title="Task Progress",
                 message="Wait and pray!",
                 cancel=True):
        self.i = -1
        self.i_max = i_max
        self.title = title
        self._message = message
        self.advance(message)

    def _log(self):
        msg = ("Progress %r: %s of %s; %r" % (self.title, self.i, self.i_max,
                                              self._message))
        logging.info(msg)

    def advance(self, new_msg=None):
        self.i += 1
        if new_msg:
            self._message = new_msg

        self._log()

    def message(self, new_msg):
        self._message = new_msg
        self._log()

    def terminate(self):
        pass


def show_help(obj):
    print getattr(obj, '__doc__', 'no docstring')



import os
import re
import webbrowser

import wx

from .._wxutils import ID, logger
from .about import AboutFrame


def wildcard(filetypes):
    if filetypes:
        return '|'.join(map('|'.join, filetypes))
    else:
        return ""


class App(wx.App):
    def OnInit(self):
        self.SetExitOnFrameDelete(False)
        self.SetAppName("Eelbrain")
        self.SetAppDisplayName("Eelbrain")

        # File Menu
        m = file_menu = wx.Menu()
        m.Append(wx.ID_OPEN, '&Open... \tCtrl+O')
        m.AppendSeparator()
        m.Append(wx.ID_CLOSE, '&Close Window \tCtrl+W')
        m.Append(wx.ID_SAVE, "Save \tCtrl+S")
        m.Append(wx.ID_SAVEAS, "Save As... \tCtrl+Shift+S")

        # Edit Menu
        m = edit_menu = wx.Menu()
        m.Append(ID.UNDO, '&Undo \tCtrl+Z')
        m.Append(ID.REDO, '&Redo \tCtrl+Shift+Z')
        m.AppendSeparator()
        m.Append(wx.ID_CUT, 'Cut \tCtrl+X')
        m.Append(wx.ID_COPY, 'Copy \tCtrl+C')
        m.Append(wx.ID_PASTE, 'Paste \tCtrl+V')
        m.AppendSeparator()
        m.Append(wx.ID_CLEAR, 'Cle&ar')

        # View Menu
        m = view_menu = wx.Menu()
        m.Append(ID.SET_VLIM, "Set Y-Axis Limit... \tCtrl+l", "Change the Y-"
                 "axis limit in epoch plots")
        m.Append(ID.SET_MARKED_CHANNELS, "Mark Channels...", "Mark specific "
                 "channels in plots.")
        m.AppendSeparator()
        m.Append(ID.SET_LAYOUT, "&Set Layout... \tCtrl+Shift+l", "Change the "
                 "page layout")
        m.AppendCheckItem(ID.PLOT_RANGE, "&Plot Data Range \tCtrl+r", "Plot "
                          "data range instead of individual sensor traces")

        # Go Menu
        m = go_menu = wx.Menu()
        m.Append(wx.ID_FORWARD, '&Forward \tCtrl+]', 'Go One Page Forward')
        m.Append(wx.ID_BACKWARD, '&Back \tCtrl+[', 'Go One Page Back')
        m.AppendSeparator()
        m.Append(ID.YIELD_TO_TERMINAL, '&Yield to Terminal \tAlt+Ctrl+Q')

        # Window Menu
        m = window_menu = wx.Menu()
        m.Append(ID.WINDOW_MINIMIZE, '&Minimize \tCtrl+M')
        m.Append(ID.WINDOW_ZOOM, '&Zoom')
        m.AppendSeparator()
        m.Append(ID.WINDOW_TILE, '&Tile')
        self.window_menu_window_items = []

        # Help Menu
        m = help_menu = wx.Menu()
        m.Append(ID.HELP_EELBRAIN, 'Eelbrain Help')
        m.Append(ID.HELP_PYTHON, "Python Help")
        m.AppendSeparator()
        m.Append(wx.ID_ABOUT, '&About Eelbrain')

        # Menu Bar
        menu_bar = wx.MenuBar()
        menu_bar.Append(file_menu, "File")
        menu_bar.Append(edit_menu, "Edit")
        menu_bar.Append(view_menu, "View")
        menu_bar.Append(go_menu, "Go")
        menu_bar.Append(window_menu, "Window")
        menu_bar.Append(help_menu, self.GetMacHelpMenuTitleName())
        wx.MenuBar.MacSetCommonMenuBar(menu_bar)
        self.menubar = menu_bar

        # Bind Menu Commands
        self.Bind(wx.EVT_MENU_OPEN, self.OnMenuOpened)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        self.Bind(wx.EVT_MENU, self.OnOpen, id=wx.ID_OPEN)
        self.Bind(wx.EVT_MENU, self.OnClear, id=wx.ID_CLEAR)
        self.Bind(wx.EVT_MENU, self.OnCloseWindow, id=wx.ID_CLOSE)
        self.Bind(wx.EVT_MENU, self.OnCopy, id=wx.ID_COPY)
        self.Bind(wx.EVT_MENU, self.OnCut, id=wx.ID_CUT)
        self.Bind(wx.EVT_MENU, self.OnOnlineHelp, id=ID.HELP_EELBRAIN)
        self.Bind(wx.EVT_MENU, self.OnOnlineHelp, id=ID.HELP_PYTHON)
        self.Bind(wx.EVT_MENU, self.OnPaste, id=wx.ID_PASTE)
        self.Bind(wx.EVT_MENU, self.OnRedo, id=ID.REDO)
        self.Bind(wx.EVT_MENU, self.OnSave, id=wx.ID_SAVE)
        self.Bind(wx.EVT_MENU, self.OnSaveAs, id=wx.ID_SAVEAS)
        self.Bind(wx.EVT_MENU, self.OnSetLayout, id=ID.SET_LAYOUT)
        self.Bind(wx.EVT_MENU, self.OnSetMarkedChannels, id=ID.SET_MARKED_CHANNELS)
        self.Bind(wx.EVT_MENU, self.OnSetVLim, id=ID.SET_VLIM)
        self.Bind(wx.EVT_MENU, self.OnTogglePlotRange, id=ID.PLOT_RANGE)
        self.Bind(wx.EVT_MENU, self.OnUndo, id=ID.UNDO)
        self.Bind(wx.EVT_MENU, self.OnWindowMinimize, id=ID.WINDOW_MINIMIZE)
        self.Bind(wx.EVT_MENU, self.OnWindowTile, id=ID.WINDOW_TILE)
        self.Bind(wx.EVT_MENU, self.OnWindowZoom, id=ID.WINDOW_ZOOM)
        self.Bind(wx.EVT_MENU, self.OnQuit, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnYieldToTerminal, id=ID.YIELD_TO_TERMINAL)

        # bind update UI
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIBackward, id=wx.ID_BACKWARD)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIClear, id=wx.ID_CLEAR)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIClose, id=wx.ID_CLOSE)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUICopy, id=wx.ID_COPY)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUICut, id=wx.ID_CUT)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIForward, id=wx.ID_FORWARD)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIOpen, id=wx.ID_OPEN)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIPaste, id=wx.ID_PASTE)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIPlotRange, id=ID.PLOT_RANGE)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIRedo, id=ID.REDO)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUISave, id=wx.ID_SAVE)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUISaveAs, id=wx.ID_SAVEAS)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUISetLayout, id=ID.SET_LAYOUT)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUISetMarkedChannels, id=ID.SET_MARKED_CHANNELS)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUISetVLim, id=ID.SET_VLIM)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUIUndo, id=ID.UNDO)

        return True

    def _get_active_frame(self):
        win = wx.Window.FindFocus()
        return wx.GetTopLevelParent(win)
        for w in wx.GetTopLevelWindows():
            if hasattr(w, 'IsActive') and w.IsActive():
                return w
        return wx.GetActiveWindow()

    def _bash_ui(self, func, *args):
        "Launch a modal dialog based on terminal input"
        if self.IsMainLoopRunning():
            logger.debug("MainLoop running")
            return func(False, *args)
        else:
            if not self.GetTopWindow():
                self.SetTopWindow(wx.Frame(None))
            wx.CallLater(10, func, True, *args)
            print "Please switch to the Python Application to provide input."
            self.MainLoop()
            return self._result

    def ask_for_dir(self, title="Select Folder", message="Please Pick a Folder",
                    must_exist=True):
        return self._bash_ui(self._ask_for_dir, title, message, must_exist)

    def _ask_for_dir(self, exit_main_loop, title, message, must_exist):
        style = wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
        if must_exist:
            style = style | wx.DD_DIR_MUST_EXIST

        dialog = wx.DirDialog(None, message, name=title,
                              style=style)
        dialog.SetTitle(title)
        if dialog.ShowModal() == wx.ID_OK:
            result = dialog.GetPath()
        else:
            result = False

        if exit_main_loop:
            self._result = result
            self.ExitMainLoop()
        else:
            return result

    def ask_for_file(self, title, message, filetypes, directory, mult):
        return self._bash_ui(self._ask_for_file, title, message, filetypes,
                             directory, mult)

    def _ask_for_file(self, exit_main_loop, title, message, filetypes,
                      directory, mult):
        """Returns path(s) or False.

        Parameters
        ----------
        ...
        directory : str
            Path to initial directory.

        Returns
        -------
        Result : list | str | None
            Paths(s) or False.
        """
        style = wx.FD_OPEN
        if mult:
            style = style | wx.FD_MULTIPLE
        dialog = wx.FileDialog(None, message, directory,
                               wildcard=wildcard(filetypes), style=style)
        dialog.SetTitle(title)
        if dialog.ShowModal() == wx.ID_OK:
            if mult:
                result = dialog.GetPaths()
            else:
                result = dialog.GetPath()
        else:
            result = False

        if exit_main_loop:
            self._result = result
            self.ExitMainLoop()
        else:
            return result

    def ask_for_string(self, title, message, default=''):
        return self._bash_ui(self._ask_for_string, title, message, default)

    def _ask_for_string(self, exit_main_loop, title, message, default):
        dlg = wx.TextEntryDialog(None, message, title, default)
        if dlg.ShowModal() == wx.ID_OK:
            result = dlg.GetValue()
        else:
            result = False

        if exit_main_loop:
            self._result = result
            self.ExitMainLoop()
        else:
            return result

    def ask_saveas(self, title, message, filetypes, defaultDir, defaultFile):
        return self._bash_ui(self._ask_saveas, title, message, filetypes,
                             defaultDir, defaultFile)

    def _ask_saveas(self, exit_main_loop, title, message, filetypes, defaultDir,
                    defaultFile):
        # setup file-dialog
        dialog = wx.FileDialog(None, message, wildcard=wildcard(filetypes),
                               style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        dialog.SetTitle(title)
        if defaultDir:
            dialog.SetDirectory(defaultDir)
        if defaultFile:
            dialog.SetFilename(defaultFile)
        # get result
        if dialog.ShowModal() == wx.ID_OK:
            result = dialog.GetPath()
        else:
            result = False

        if exit_main_loop:
            self._result = result
            self.ExitMainLoop()
        else:
            return result

    def message_box(self, message, caption, style):
        return self._bash_ui(self._message_box, message, caption, style)

    def _message_box(self, exit_main_loop, message, caption, style):
        result = wx.MessageBox(message, caption, style)

        if exit_main_loop:
            self._result = result
            self.ExitMainLoop()
        else:
            return result

    def OnAbout(self, event):
        if hasattr(self, '_about_frame') and hasattr(self._about_frame, 'Raise'):
            self._about_frame.Raise()
        else:
            self._about_frame = AboutFrame(None)
            self._about_frame.Show()
#             frame.SetFocus()

    def OnClear(self, event):
        frame = self._get_active_frame()
        frame.OnClear(event)

    def OnCloseWindow(self, event):
        frame = self._get_active_frame()
        logger.debug("Close %r" % frame)
        if frame is not None:
            frame.Close()

    def OnCopy(self, event):
        win = wx.Window.FindFocus()
        logger.debug("Copy %r" % win)
        win.Copy()
#
    def OnCut(self, event):
        win = wx.Window.FindFocus()
        logger.debug("Cut %r" % win)
        win.Cut()

    def OnMenuOpened(self, event):
        "Updates window names in the window menu"
        menu = event.GetMenu()
        if menu.GetTitle() == 'Window':
            # clear old entries
            while self.window_menu_window_items:
                item = self.window_menu_window_items.pop()
                menu.RemoveItem(item)
                self.Unbind(wx.EVT_MENU, id=item.GetId())
            # add new entries
            for window in wx.GetTopLevelWindows():
                id_ = window.GetId()
                if id_ < 0:
                    continue
                item = menu.Append(id_, window.GetTitle())
                self.Bind(wx.EVT_MENU, self.OnWindowRaise, id=id_)
                self.window_menu_window_items.append(item)

    def OnOnlineHelp(self, event):
        "Called from the Help menu to open external resources"
        Id = event.GetId()
        if Id == ID.HELP_EELBRAIN:
            webbrowser.open("https://pythonhosted.org/eelbrain/")
        elif Id == ID.HELP_PYTHON:
            webbrowser.open("http://docs.python.org/2.7/")
        else:
            raise RuntimeError("Invalid help ID")

    def OnOpen(self, event):
        frame = self._get_active_frame()
        frame.OnOpen(event)

    def OnPaste(self, event):
        win = wx.Window.FindFocus()
        logger.debug("Paste %r" % win)
        win.Paste()

    def OnQuit(self, event):
        logger.debug("OnQuit %s", event)
        for win in wx.GetTopLevelWindows():
            if not win.Close():
                return
        self.ExitMainLoop()

    def OnRedo(self, event):
        frame = self._get_active_frame()
        frame.OnRedo(event)

    def OnSave(self, event):
        frame = self._get_active_frame()
        frame.OnSave(event)

    def OnSaveAs(self, event):
        frame = self._get_active_frame()
        frame.OnSaveAs(event)

    def OnSetVLim(self, event):
        frame = self._get_active_frame()
        frame.OnSetVLim(event)

    def OnSetLayout(self, event):
        frame = self._get_active_frame()
        frame.OnSetLayout(event)

    def OnSetMarkedChannels(self, event):
        frame = self._get_active_frame()
        frame.OnSetMarkedChannels(event)

    def OnTogglePlotRange(self, event):
        frame = self._get_active_frame()
        frame.OnTogglePlotRange(event)

    def OnUndo(self, event):
        frame = self._get_active_frame()
        frame.OnUndo(event)

    def OnUpdateUIBackward(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIBackward'):
            frame.OnUpdateUIBackward(event)
        else:
            event.Enable(False)

    def OnUpdateUIClear(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIClear'):
            frame.OnUpdateUIClear(event)
        else:
            event.Enable(False)

    def OnUpdateUIClose(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIClose'):
            frame.OnUpdateUIClose(event)
        else:
            event.Enable(False)

    def OnUpdateUICopy(self, event):
        win = wx.Window.FindFocus()
        event.Enable(win and hasattr(win, 'CanCopy') and win.CanCopy())

    def OnUpdateUICut(self, event):
        win = wx.Window.FindFocus()
        event.Enable(win and hasattr(win, 'CanCut') and win.CanCut())

    def OnUpdateUIForward(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIForward'):
            frame.OnUpdateUIForward(event)
        else:
            event.Enable(False)

    def OnUpdateUIOpen(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIOpen'):
            frame.OnUpdateUIOpen(event)
        else:
            event.Enable(False)

    def OnUpdateUIPaste(self, event):
        win = wx.Window.FindFocus()
        event.Enable(win and hasattr(win, 'CanPaste') and win.CanPaste())

    def OnUpdateUIPlotRange(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIPlotRange'):
            frame.OnUpdateUIPlotRange(event)
        else:
            event.Check(False)
            event.Enable(False)

    def OnUpdateUIRedo(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIRedo'):
            frame.OnUpdateUIRedo(event)
        else:
            event.Enable(False)

    def OnUpdateUISave(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISave'):
            frame.OnUpdateUISave(event)
        else:
            event.Enable(False)

    def OnUpdateUISaveAs(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISaveAs'):
            frame.OnUpdateUISaveAs(event)
        else:
            event.Enable(False)

    def OnUpdateUISetLayout(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetLayout'):
            frame.OnUpdateUISetLayout(event)
        else:
            event.Enable(False)

    def OnUpdateUISetMarkedChannels(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetMarkedChannels'):
            frame.OnUpdateUISetMarkedChannels(event)
        else:
            event.Enable(False)

    def OnUpdateUISetVLim(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUISetVLim'):
            frame.OnUpdateUISetVLim(event)
        else:
            event.Enable(False)

    def OnUpdateUIUndo(self, event):
        frame = self._get_active_frame()
        if frame and hasattr(frame, 'OnUpdateUIUndo'):
            frame.OnUpdateUIUndo(event)
        else:
            event.Enable(False)

    def OnWindowMinimize(self, event):
        frame = self._get_active_frame()
        if frame:
            frame.Iconize()

    def OnWindowRaise(self, event):
        id_ = event.GetId()
        window = wx.FindWindowById(id_)
        window.Raise()

    def OnWindowTile(self, event):
        frames = sorted(wx.GetTopLevelWindows(), lambda x, y: x.Position[0] < y.Position[0])
        dx, dy = wx.DisplaySize()
        x = 0
        y = 0
        y_next = 0
        for frame in frames:
            sx, sy = frame.Size
            if x and x + sx > dx:
                if y_next > dy:
                    return
                x = 0
                y = y_next
            frame.Position = (x, y)
            y_next = max(y_next, y + sy)
            x += sx

    def OnWindowZoom(self, event):
        frame = self._get_active_frame()
        if frame:
            frame.Maximize()

    def OnYieldToTerminal(self, event):
        self.ExitMainLoop()


def get_app():
    app = wx.GetApp()
    if app is None or not isinstance(app, App):
        logger.debug("Initializing Eelbrain App")
        app = App()
    return app


def run():
    app = get_app()
    if not app.IsMainLoopRunning():
        print "Starting GUI. Quit the Python application to return to the shell..."
        app.MainLoop()



import logging

import wx


def filename_repr(filenames):
    if filenames:
        if isinstance(filenames, basestring):
            filenames = [filenames]
        
#        if 'wxMSW' in wx.PlatformInfo:
            # does windows require separate handling for to backslash?
            # on os-x, if the path contains a backslash, it is inserted as '\\'
#            conv = []
#            for name in filenames:
#                try:
#                    conv.append('r%r' % str(name))
#                except:
#                    conv.append(repr(name))
#            
#            if len(filenames) == 1:
#                string = filenames[0]
#            else:
#                string = '[' + ', '.join(filenames) + ']'
#        
#        else:
        try:
            filenames = map(str, filenames)
        except:
            pass
        
        logging.debug(filenames)
        
        if len(filenames) == 1:
            string = repr(filenames[0])
        else:
            string = repr(filenames)
        
        return string
    else:
        return ''



class FilenameDropTarget(wx.FileDropTarget):
    """
    File drop target: http://wiki.wxpython.org/DragAndDrop
    
    (!) apparently the FileDropTarget can only be used once; when I saved the 
    instance as an Editor attribute and SetDropTarget it to a second editor
    this caused a segmentation fault.
    
    """
    def __init__(self, text_target):
        wx.FileDropTarget.__init__(self)
        self.text_target = text_target
    
    def OnDropFiles(self, x, y, filenames):
        msg = "DROP! %r" % filenames
        logging.info(msg)
        if len(filenames) == 1:
            filenames = "'%s'"%(filenames[0])
        self.text_target.ReplaceSelection(str(filenames))


class TextDropTarget(wx.TextDropTarget):
    def __init__(self, text_target):
        wx.TextDropTarget.__init__(self)
        self.text_target = text_target
    
    def OnDropText(self, x, y, text):
        msg = "DROP! %r" % text
        logging.info(msg)
        self.text_target.ReplaceSelection(text)


class StringDropTarget(wx.DropTarget):
    """
    DropTarget for multiple data types based on:
    http://www.wiki.wxpython.org/DragAndDrop#wxDataObjectComposite
    
    """
    def __init__(self, target):
        wx.DropTarget.__init__(self)
        self.target = target
        
        self.do = wx.DataObjectComposite()  # the dataobject that gets filled with the appropriate data
        self.filedo = wx.FileDataObject()
        self.textdo = wx.TextDataObject()
#        self.bmpdo = wx.BitmapDataObject()
        self.do.Add(self.filedo)
        self.do.Add(self.textdo)
#        self.do.Add(self.bmpdo)
        self.SetDataObject(self.do)
        
    def OnData(self, x, y, d):#, data):
        if self.GetData():
            df = self.do.GetReceivedFormat().GetType()
#            data = self.GetData()
            
            if df in [wx.DF_UNICODETEXT, wx.DF_TEXT]:
                string = self.textdo.GetText()
            elif df == wx.DF_FILENAME:
                filenames = self.filedo.GetFilenames()
                string = filename_repr(filenames)
            
            msg = "OnData! %r" % string   
            logging.info(msg)
            self.target.ReplaceSelection(string)
        return d
    


def set_for_strings(window):
    drop_target = StringDropTarget(window)
    window.SetDropTarget(drop_target)
    




'''
Plot sensor maps.
'''
# author: Christian Brodbeck


import os

import numpy as np
import matplotlib as mpl
from matplotlib.lines import Line2D

from .._data_obj import Datalist, as_sensor
from ._base import _EelFigure


# some useful kwarg dictionaries for different plot layouts
kwargs_mono = dict(mc='k',
                   lc='.5',
                   hllc='k',
                   hlmc='k',
                   hlms=7,
                   strlc='k')


class _plt_connectivity:
    def __init__(self, ax, locs, connectivity, linestyle={}):
        self.ax = ax
        self.locs = locs
        self._h = []
        self.show(connectivity, linestyle)

    def show(self, connectivity, linestyle={}):
        while self._h:
            self._h.pop().remove()

        if connectivity is None:
            return

        for c, r in connectivity:
            x = self.locs[[c, r], 0]
            y = self.locs[[c, r], 1]
            line = Line2D(x, y, **linestyle)
            self.ax.add_line(line)
            self._h.append(line)


class _ax_map2d:
    def __init__(self, ax, sensors, proj='default', extent=1,
                 frame=.02, kwargs=None):
        self.ax = ax

        if kwargs is None:
            kwargs = {'marker': 'x',  # symbol
                      'color': 'b',  # mpl plot kwargs ...
                      'ms': 3,  # marker size
                      'markeredgewidth': .5,
                      'ls': ''}

        ax.set_aspect('equal')
        # ax.set_frame_on(False)
        ax.set_axis_off()

        h = _plt_map2d(ax, sensors, proj, extent=extent, kwargs=kwargs)
        self.sensors = h

        locs = sensors.get_locs_2d(proj=proj, extent=extent)
        self.connectivity = _plt_connectivity(ax, locs, None)

        ax.set_xlim(-frame, 1 + frame)

    def remove(self):
        "remove from axes"
        self.sensors.remove()


class _plt_map2d:

    def __init__(self, ax, sensors, proj='default', extent=1, mark=None,
                 labels=None, kwargs=None):
        """
        Parameters
        ----------
        ax : matplotlib Axes
            Axes.
        sensors : Sensor
            Sensor dimension.

        labels : None | 'index' | 'name' | 'fullname'
            Content of the labels. For 'name', any prefix common to all names
            is removed; with 'fullname', the full name is shown.
        """
        self.ax = ax
        self.sensors = sensors
        self.locs = sensors.get_locs_2d(proj=proj, extent=extent)
        self._mark_handles = None

        if kwargs is None:
            kwargs = {'marker': '.',  # symbol
                      'color': 'k',  # mpl plot kwargs ...
                      'ms': 1,  # marker size
                      'markeredgewidth': .5,
                      'ls': ''}

        if mark is not None:
            mark_idx = sensors.dimindex(mark)
            locs = self.locs[mark_idx]
        else:
            locs = self.locs
            mark_idx = None

        self.mark = mark
        self._mark_idx = mark_idx
        self.markers = []
        self.labels = []
        if labels is not None:
            self.show_labels(labels)

        if 'color' in kwargs:
            h = ax.plot(locs[:, 0], locs[:, 1], **kwargs)
            self.markers += h
        else:
            colors = mpl.rcParams['axes.color_cycle']
            nc = len(colors)
            for i in xrange(len(locs)):
                kwargs['color'] = kwargs['mec'] = colors[i % nc]
                h = ax.plot(locs[i, 0], locs[i, 1], **kwargs)
                self.markers += h

    def mark_sensors(self, sensors, *args, **kwargs):
        """Mark specific sensors

        Parameters
        ----------
        sensors : None | Sensor dimension index
            Sensors which should be marked (None to clear all markings).
        others :
            Matplotlib :func:`pyplot.scatter` parameters for the marking
            sensors.
        """
        if sensors is None:
            while self._mark_handles:
                self._mark_handles.pop().remove()
            return
        elif not np.count_nonzero(sensors):
            return

        idx = self.sensors.dimindex(sensors)
        locs = self.locs[idx]
        self._mark_handles = self.ax.scatter(locs[:, 0], locs[:, 1], *args, **kwargs)

    def remove(self):
        "remove from axes"
        while self._mark_handles:
            self._mark_handles.pop().remove()
        while self.markers:
            self.markers.pop().remove()

    def show_labels(self, text='name', xpos=0, ypos=.01, **text_kwargs):
        """Plot labels for the sensors

        Parameters
        ----------
        text : None | 'index' | 'name' | 'fullname'
            Content of the labels. For 'name', any prefix common to all names
            is removed; with 'fullname', the full name is shown.
        xpos, ypos : scalar
            The position offset of the labels from the sensor markers.
        text_kwargs : **
            Matplotlib text parameters.
        """
        # remove existing labels
        while self.labels:
            self.labels.pop().remove()

        if not text:
            return

        kwargs = dict(color='k', fontsize=8, horizontalalignment='center',
                      verticalalignment='bottom')
        kwargs.update(text_kwargs)

        sensors = self.sensors
        if text == 'index':
            labels = map(str, xrange(len(sensors)))
        elif text == 'name':
            labels = sensors.names
            prefix = os.path.commonprefix(labels)
            pf_len = len(prefix)
            labels = [label[pf_len:] for label in labels]
        elif text == 'fullname':
            labels = sensors.names
        else:
            err = "text has to be 'index' or 'name', can't be %r" % text
            raise NotImplementedError(err)

        locs = self.locs
        if self._mark_idx is not None:
            labels = Datalist(labels)[self._mark_idx]
            locs = locs[self._mark_idx]

        locs = locs + [[xpos, ypos]]
        for loc, txt in zip(locs, labels):
            x , y = loc
            h = self.ax.text(x, y, txt, **kwargs)
            self.labels.append(h)

    def set_label_color(self, color='k'):
        """Change the color of all sensor labels

        Parameters
        ----------
        color : matplotlib color
            New color for the sensor labels.
        """
        for h in self.labels:
            h.set_color(color)


class SensorMapMixin:
    # expects self._sensor_plots to be list of _plt_map2d
    __label_options = ['None', 'Index', 'Name', 'Full Name']
    __label_option_args = [None, 'index', 'name', 'fullname']

    def __init__(self, sensor_plots, label=None):
        """Call after EelFigure init (toolbar fill)

        Parameters
        ----------
        sensor_plots : list of _plt_map2d
            Sensor-map objects.
        label : None | str
            Initial label argument (default None).
        """
        self.__label_color = 'k'
        self.__check_label_arg(label)
        self.__sensor_plots = sensor_plots
        self.__LabelChoice.SetSelection(self.__label_option_args.index(label))

    def _fill_toolbar(self, tb):
        import wx
        tb.AddSeparator()

        # sensor labels
        lbl = wx.StaticText(tb, -1, "Labels:")
        tb.AddControl(lbl)
        choice = wx.Choice(tb, -1, choices=self.__label_options)
        tb.AddControl(choice)
        self.__LabelChoice = choice
        choice.Bind(wx.EVT_CHOICE, self.__OnSensorLabelChoice)

        # sensor label color
        choices = ['black', 'white', 'blue', 'green', 'red', 'cyan', 'magenta',
                   'yellow']
        choice = wx.Choice(tb, -1, choices=choices)
        tb.AddControl(choice)
        self.__LabelColorChoice = choice
        choice.Bind(wx.EVT_CHOICE, self.__OnSensorLabelColorChoice)

        btn = wx.Button(tb, label="Mark")  # , style=wx.BU_EXACTFIT)
        btn.Bind(wx.EVT_BUTTON, self.__OnMarkSensor)
        tb.AddControl(btn)

    def __check_label_arg(self, arg):
        if arg not in self.__label_option_args:
            raise ValueError("Invalid sensor label argument: %s" % repr(arg))

    def __OnMarkSensor(self, event):
        import wx
        msg = "Channels to mark, separated by comma"
        dlg = wx.TextEntryDialog(self._frame, msg, "Mark Sensor")
        if dlg.ShowModal() != wx.ID_OK:
            return

        chs = filter(None, map(unicode.strip, dlg.GetValue().split(',')))
        try:
            self.mark_sensors(chs)
        except Exception as exc:
            msg = '%s: %s' % (type(exc).__name__, exc)
            sty = wx.OK | wx.ICON_ERROR
            wx.MessageBox(msg, "Mark Sensors Failed for %r" % chs, style=sty)

    def __OnSensorLabelChoice(self, event):
        sel = event.GetSelection()
        sel_arg = self.__label_option_args[sel]
        self.set_label_text(sel_arg)

    def __OnSensorLabelColorChoice(self, event):
        sel = event.GetSelection()
        color = ['k', 'w', 'b', 'g', 'r', 'c', 'm', 'y'][sel]
        self.set_label_color(color)

    def mark_sensors(self, sensors, *args, **kwargs):
        """Mark given sensors on the plots

        Parameters
        ----------
        sensors : None | Sensor dimension index
            Sensors which should be marked (None to clear all markings).
        s : scalar | sequence of scalars
            Marker size(s) in points^2.
        c : color | sequence of colors
            Marker color(s).
        marker : str
            Marker style, default: ``'o'``.

        (Matplotlib :func:`pyplot.scatter` parameters)
        """
        for p in self.__sensor_plots:
            p.mark_sensors(sensors, *args, **kwargs)
        self.draw()

    def set_label_color(self, color='w'):
        if hasattr(self, '_SensorLabelChoice'):
            sels = ['k', 'w', 'b', 'g', 'r', 'c', 'm', 'y']
            if color in sels:
                sel = sels.index(color)
                self.__LabelColorChoice.SetSelection(sel)

        self.__label_color = color
        for p in self.__sensor_plots:
            p.set_label_color(color)
        self.draw()

    def set_label_text(self, text='name'):
        """Add/remove sensor labels

        Parameters
        ----------
        labels : None | 'name' | 'index'
            Content of the labels. For 'name', any prefix common to all names
            is removed; with 'fullname', the full name is shown.
        """
        self.__check_label_arg(text)
        if hasattr(self, '_SensorLabelChoice'):
            sel = self.__label_option_args.index(text)
            self.__LabelChoice.SetSelection(sel)

        for p in self.__sensor_plots:
            p.show_labels(text, color=self.__label_color)
        self.draw()


class SensorMaps(_EelFigure):
    """Multiple views on a sensor layout.

    Allows selecting sensor groups and retrieving corresponding indices.

    Parameters
    ----------
    sensors : Sensor | NDVar
        The sensors to use, or an NDVar with a sensor dimension.
    select : list of int
        Initial selection.
    proj : str
        Sensor projection for the fourth plot.
    frame : scalar
        Size of the empty space around sensors in axes.
    title : None | string
        Figure title.

    Notes
    -----
    **Selecting Sensor Groups:**

     - Dragging with the left mouse button adds sensors to the selection.
     - Dragging with the right mouse button removes sensors from the current
       selection.
     - The 'Clear' button (or :meth:`clear`) clears the selection.

    """
    def __init__(self, sensors, select=[], proj='default', frame=0.05,
                 *args, **kwargs):
        sensors = as_sensor(sensors)

        # layout figure
        ftitle = 'SensorMaps'
        sens_name = getattr(sensors, 'sysname', None)
        if sens_name:
            ftitle = '%s: %s' % (ftitle, sens_name)

        self._drag_ax = None
        self._drag_x = None
        self._drag_y = None
        _EelFigure.__init__(self, ftitle, 4, 3, 1, False, ncol=2, nrow=2, *args,
                            **kwargs)
        self.figure.subplots_adjust(left=0, bottom=0, right=1, top=1,
                                    wspace=.1, hspace=.1)

        # store args
        self._sensors = sensors

        ext = np.vstack((sensors.locs.min(0), sensors.locs.max(0)))
        aframe = np.array([-frame, frame])
        xlim = ext[:, 0] + aframe
        ylim = ext[:, 1] + aframe
        zlim = ext[:, 2] + aframe

        # back
        ax = self.ax0 = self.figure.add_subplot(2, 2, 1)
        ax.proj = 'y-'
        ax.extent = False
        self._h0 = _ax_map2d(ax, sensors, proj=ax.proj, extent=ax.extent)

        # left
        ax = self.ax1 = self.figure.add_subplot(2, 2, 2, sharey=self.ax0)
        ax.proj = 'x-'
        ax.extent = False
        self._h1 = _ax_map2d(ax, sensors, proj=ax.proj, extent=ax.extent)

        # top
        ax = self.ax2 = self.figure.add_subplot(2, 2, 3, sharex=self.ax0)
        ax.proj = 'z+'
        ax.extent = False
        self._h2 = _ax_map2d(ax, sensors, proj=ax.proj, extent=ax.extent)

        self.ax0.set_xlim(*xlim)
        self.ax0.set_ylim(*zlim)
        self.ax1.set_xlim(*zlim)
        self.ax2.set_ylim(*ylim)

        # proj
        ax = self.ax3 = self.figure.add_subplot(2, 2, 4)
        ax.proj = proj
        ax.extent = 1
        self._h3 = _ax_map2d(ax, sensors, proj=ax.proj, extent=ax.extent)
        self.ax3.set_xlim(-frame, 1 + frame)
        self.ax3.set_ylim(-frame, 1 + frame)

        self._sensor_maps = (self._h0, self._h1, self._h2, self._h3)
        self._show()

        # selection
        self.sel_kwargs = dict(marker='o', s=5, c='r', linewidths=.9)
        self._sel_h = []
        if select is not None:
            self.set_selection(select)
        else:
            self.select = None

        # setup mpl event handling
        self.canvas.mpl_connect("button_press_event", self._on_button_press)
        self.canvas.mpl_connect("button_release_event", self._on_button_release)

    def _fill_toolbar(self, tb):
        import wx

        tb.AddSeparator()

        # plot labels
        btn = wx.Button(tb, wx.ID_CLEAR, "Clear")
        tb.AddControl(btn)
        btn.Bind(wx.EVT_BUTTON, self._OnClear)

    def clear(self):
        "Clear the current sensor selection."
        self.select = None
        self.update_mark_plot()

    def get_selection(self):
        """
        Returns
        -------
        selection : list
            Returns the current selection as a list of indices.
        """
        if self.select is None:
            return []
        else:
            return np.where(self.select)[0]

    def _on_button_press(self, event):
        ax = event.inaxes
        if ax:
            self._is_dragging = True
            self._drag_ax = event.inaxes
            self._drag_x = event.xdata
            self._drag_y = event.ydata

            self.canvas.store_canvas()
            x = np.ones(5) * event.xdata
            y = np.ones(5) * event.ydata
            self._drag_rect = ax.plot(x, y, '-k')[0]

    def _on_button_release(self, event):
        if not hasattr(self, '_drag_rect'):
            return

        x = self._drag_rect.get_xdata()
        y = self._drag_rect.get_ydata()
        xmin = min(x)
        xmax = max(x)
        ymin = min(y)
        ymax = max(y)

        ax = self._drag_ax
        locs = self._sensors.get_locs_2d(ax.proj, extent=ax.extent)
        x = locs[:, 0]
        y = locs[:, 1]
        sel = (x > xmin) & (x < xmax) & (y > ymin) & (y < ymax)

        if self.select is None:
            self.select = sel
        elif event.button == 1:
            self.select[sel] = True
        else:
            self.select[sel] = False

        # clear dragging-related attributes
        self._drag_rect.remove()
        del self._drag_rect
        self._drag_ax = None
        self._drag_x = None
        self._drag_y = None

        self.update_mark_plot()

    def _on_motion(self, event):
        super(self.__class__, self)._on_motion(event)
        ax = event.inaxes
        if ax and ax is self._drag_ax:
            x0 = self._drag_x
            x1 = event.xdata
            y0 = self._drag_y
            y1 = event.ydata
            x = [x0, x1, x1, x0, x0]
            y = [y0, y0, y1, y1, y0]
            self._drag_rect.set_data(x, y)
            self.canvas.redraw(artists=[self._drag_rect])

    def _OnClear(self, event):
        self.clear()

    def set_selection(self, select):
        """
        Set the current selection with a list of indices.

        Parameters
        ----------
        select : sensor index
            Index for sensor dimension, for example array_like of int, or list
            of sensor names.
        """
        idx = self._sensors.dimindex(select)
        self.select = np.zeros(len(self._sensors), dtype=bool)
        self.select[idx] = True
        self.update_mark_plot()

    def update_mark_plot(self):
        for h in self._sensor_maps:
            h.sensors.mark_sensors(self.select, **self.sel_kwargs)
        self.canvas.draw()


class SensorMap(SensorMapMixin, _EelFigure):
    """Plot sensor positions in 2 dimensions

    Parameters
    ----------
    sensors : NDVar | Sensor
        sensor-net object or object containing sensor-net
    labels : None | 'index' | 'name' | 'fullname'
        Content of the labels. For 'name', any prefix common to all names
        is removed; with 'fullname', the full name is shown.
    proj:
        Transform to apply to 3 dimensional sensor coordinates for plotting
        locations in a plane
    mark : None | list of int
        List of sensor indices to mark.
    frame : scalar
        Size of the empty space around sensors in axes.
    connectivity : bool
        Show sensor connectivity (default False).
    title : None | string
        Figure title.
    """
    def __init__(self, sensors, labels='name', proj='default', mark=None,
                 frame=.05, connectivity=False, *args, **kwargs):
        sensors = as_sensor(sensors)

        if sensors.sysname:
            ftitle = 'SensorMap: %s' % sensors.sysname
        else:
            ftitle = 'SensorMap'
        _EelFigure.__init__(self, ftitle, 1, 7, 1, False, *args, **kwargs)
        self.axes = self._axes[0]

        # store args
        self._sensors = sensors
        self._proj = proj
        self._marker_handles = []
        self._connectivity = None

        self._markers = _ax_map2d(self.axes, sensors, proj, frame=frame)
        SensorMapMixin.__init__(self, [self._markers.sensors])

        if labels:
            self.set_label_text(labels)
        if mark is not None:
            self.mark_sensors(mark)

        if connectivity:
            self.show_connectivity()

        self._show()

    def mark_sensors(self, mark, kwargs=dict(marker='o',  # symbol
                                             color='r',  # mpl plot kwargs ...
                                             ms=5,  # marker size
                                             markeredgewidth=.9,
                                             ls='',
                                             )):
        """Mark specific sensors.

        Parameters
        ----------
        mark : sequence of {str | int}
            List of sensor names or indices.
        kwargs : dict
            Dict with kwargs for customizing the sensor plot (matplotlib plot
            kwargs).

        See Also
        --------
        .remove_markers() : Remove the markers
        """
        h = _plt_map2d(self.axes, self._sensors, self._proj, mark=mark,
                       kwargs=kwargs)
        self._marker_handles.append(h)
        self.canvas.draw()

    def remove_markers(self):
        "Remove all sensor markers."
        while len(self._marker_handles) > 0:
            h = self._marker_handles.pop(0)
            h.remove()
        self.canvas.draw()

    def show_connectivity(self, show=True):
        """Show the sensor connectivity as lines connecting sensors.

        Parameters
        ----------
        show : None | True | scalar
            True to show the default connectivity.
            None to remove the connectivity lines.
            Scalar to plot connectivity for a different connect_dist parameter
            (see Sensor.connectivity()).
        """
        if not show:
            self._markers.connectivity.show(None)
        else:
            if show is True:
                conn = self._sensors.connectivity()
            else:
                conn = self._sensors.connectivity(show)
            self._markers.connectivity.show(conn)
        self.draw()


def map3d(sensors, marker='c*', labels=False, head=0):
    """3d plot of a Sensors instance"""
    import matplotlib.pyplot as plt

    sensors = as_sensor(sensors)

    locs = sensors.locs
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(locs[:, 0], locs[:, 1], locs[:, 2])
    # plot head ball
    if head:
        u = np.linspace(0, 1 * np.pi, 10)
        v = np.linspace(0, np.pi, 10)

        x = 5 * head * np.outer(np.cos(u), np.sin(v))
        z = 10 * (head * np.outer(np.sin(u), np.sin(v)) - .5)  # vertical
        y = 5 * head * np.outer(np.ones(np.size(u)), np.cos(v))  # axis of the sphere
        ax.plot_surface(x, y, z, rstride=1, cstride=1, color='w')
    # n = 100
    # for c, zl, zh in [('r', -50, -25), ('b', -30, -5)]:
    # xs, ys, zs = zip(*
    #               [(random.randrange(23, 32),
    #                 random.randrange(100),
    #                 random.randrange(zl, zh)
    #                 ) for i in range(n)])
    # ax.scatter(xs, ys, zs, c=c)
